from sudoku_variants.rule.interface import Rule, WithData
from sudoku_variants.rule.orthogonal import Orthogonal
from sudoku_variants.rule.sub_board import SubBoard
from sudoku_variants.rule.knight import Knight
from sudoku_variants.rule.king import King
from sudoku_variants.rule.consecutive import Consecutive
from sudoku_variants.rule.jigsaw import Jigsaw
